%% Using statistics_crossvalidate together with FieldTrip data
% This example demonstrates how to use neuroimaging data obtained from FieldTrip
% together with the statistics_crossvalidate function. In the example, we make use of covert
% attention data of one subject that has already been frequency analyzed.
%
% The data consists of 7 different frequencies at 274 channels at time
% points [-0.5 0 0.5 1 1.5 2 2.5]. We can expect evoked response after the
% cue and alpha modulation after about 1 second.
%
% Copyright (C) 2008  Marcel van Gerven
%

%% Classification examples using FieldTrip's statistics_crossvalidate
function statistics_crossvalidate_demo()

%% 
% Some initialization

fclose('all');
close all
clear all
format long

% fix the RNG in order to reproduce the experiment
rand('twister',1); randn('state',1);

%% 
% load data
load ~/code/classification/toolboxes/bayesbrain/examples/freqli;
load ~/code/classification/toolboxes/bayesbrain/examples/freqri;
data = {freqLI freqRI}; clear freqLI; clear freqRI;

%%
% initialize parameters

cfg = [];
cfg.channel     = {'MLO' 'MRO'};
cfg.frequency   = [8 30];
cfg.latency     = [0.5 2.5];
cfg.avgovertime = 'yes';

%%
% create design matrix; this specifies at least the class labels
% for each example
cfg.design = [];
for c=1:length(data)
    cfg.design = [cfg.design; c*ones(size(data{c}.powspctrm,1),1)];
end

%% 
% specify crossvalidate as statistical procedure and use discriminant
% analysis as a classifier
cfg.method = 'crossvalidate';
cfg.classifier = da();

%% 
% call crossvalidation through freqstatistics
stat = freqstatistics(cfg,data{:});

%%
% output classification rate

stat.metric

%% 
% Look at posteriors of the first fold

stat.post{1}

%% 
% Look at contents of the classification procedure

% the third method of the classification procedure for the first fold
stat.procedure{1}.clfmethods{3}


%% Classification using a different classifier

% logistic regression instead of discriminant analysis

cfg.classifier = lr();
stat = freqstatistics(cfg,data{:});

%%
% note the better performance
stat.metric

%% Using a different number of folds and a different metric

cfg.metric = 'auc'; % area under the ROC curve
cfg.cvfolds = 0.8; % use 80% for training and 20% for testing
stat = freqstatistics(cfg,data{:});
stat.metric

%%
% for this classifier we can project parameters back onto the scalp

% get the structure 
load ~/code/classification/toolboxes/bayesbrain/examples/freqli;

% get model in the first fold
model = full(stat.procedure.clfmethods{3}.model(1,1:(end-1)));

% change dimensions
freqLI.time = 1;
freqLI.freq = [8 12 20 30];
freqLI.time = 1;
freqLI.powspctrm = zeros(274,4,1);
freqLI.dimord = 'chan_freq_time';

% set powerspectrum to parameters
channels = ismember(freqLI.label,channelselection({'MLO' 'MRO'},freqLI.label));
freqLI.powspctrm(channels,:,:) = reshape(model,size(freqLI.powspctrm(channels,:,:)));

% predictive map
tcfg.layout = 'CTF274.lay';
topoplotTFR(tcfg,freqLI);

end