classdef clfproc
%clfproc classification procedure class
%   
%   A classification procedure is just a sequence of classification methods {method1 method2 method3 ...} that are
%   called in this order and where the output of the previous method acts
%   as input to the next method.
%
%   A classification method always has a train and test function for the
%   two different modes of operation. A classification procedure is
%   just a sequence of methods {method1 method2 method3 ...} that are
%   called in this order and where the output of the previous method acts
%   as input to the next method. Note that method derives from handle
%   so any changes are permanent. Methods:
%
%   PREPROCESSOR: produces transformed data
%   FEATURESELECTOR: selects and/or extracts features and produces data
%   CLASSIFIER: produces posterior probabilities
%
%   the output of a classifier can be used as input to:
%
%   evaluate: takes posteriors and labels and produces an evaluation measure
%
%   parameters of a method should be set by means of a constructor call
%   method(obj,varargin)
%
%   repeated calls to train should allow for online learning as much as
%   possible
%   
%   data is given by an N x M matrix with N examples/times and 
%   M features
%   design is an N x 1 matrix with the N class labels 1,2,3,...
%
%   multiple datasets are entered using cell arrays of data/design
%
%   Copyright (c) 2008, Marcel van Gerven
%
%   $Log: clfproc.m,v $
%

    properties

        clfmethods; % the methods that specify the classification procedure
        
    end
    
    methods
       function obj = clfproc(clfmethods)
       % constructor expects classification methods
       
        if ~nargin
            % if unspecified use a random classifier
            clfmethods = { randomclassifier() };
        end

        if ~isempty(clfmethods)

            % check if this is a valid classification procedure:
            assert(all(cellfun(@(x)(isa(x,'preprocessor') || isa(x,'featureselector') || ...
                isa(x,'classifier')),clfmethods)));

            % at least one classification method at the end
            assert(isa(clfmethods{end},'classifier'));
        end

        obj.clfmethods = clfmethods;
       
       end
       function obj = train(obj,data,design)
            % train just calls the methods' train functions in order to produce a posterior

            % make sure data is a (cell array of) matrix
            if (iscell(data) && ndims(data{1}) > 2) || ndims(data) > 2
                data = collapse(data);
            end
            
            for c=1:length(obj.clfmethods)
      
                obj.clfmethods{c} = obj.clfmethods{c}.train(data,design);
                data = obj.clfmethods{c}.test(data);                
            end        
                   
       end
       function data = test(obj,data)
           % test just calls the methods' test functions in order to produce a posterior
           
           % make sure data is a (cell array of) matrix
           if (iscell(data) && ndims(data{1}) > 2) || ndims(data) > 2
               data = collapse(data);
           end

           for c=1:length(obj.clfmethods)
               
               data = obj.clfmethods{c}.test(data);
           end

       end       
       
    end
end 

