classdef crossvalidator < validator
%CROSSVALIDATOR crossvalidation class
%
%   OPTIONS:
%   cvfolds: the type of crossvalidation used:
%       - 1 : test on training data
%       - 0 <= cvfolds < 1 : proportion of training data
%       - cvfolds = Inf : leave-one-out crossvalidation
%       - cvfolds = N  : N-fold crossvalidation
%
%   SEE ALSO:
%   loocrossvalidator.m
%
%   Copyright (c) 2008, Marcel van Gerven
%
%   $Log: crossvalidator.m,v $
%

    properties
        cvfolds = 10; % crossvalidation procedure
    end

   methods
       function obj = crossvalidator(varargin)
           
           obj = obj@validator(varargin{:});
                       
       end
       function obj = validate(obj,data,design)

           % make sure data is a (cell array of) matrix
           if (iscell(data) && ndims(data{1}) > 2) || ndims(data) > 2
               data = obj.collapse(data);
           end
           
           % randomize ordering of the data before assigning folds
           if obj.randomize
               [data,design] = obj.shuffle(data,design);
           end

           cvfolds = obj.cvfolds;
           
           if cvfolds == 1 % test on training data

               if obj.verbose, fprintf('validating using training data for testing\n'); end
        
               if (islogical(obj.balanced) && obj.balanced) || strcmp(obj.balanced,'train') || strcmp(obj.balanced,'test')
                   if obj.verbose, fprintf('balancing data\n'); end
                   [data,design] = obj.balance(data,design);
               end

               assert(~isempty(data));
                  
               obj.procedure = obj.procedure.train(data,design);
               obj.post = obj.procedure.test(data);
               obj.design = design;
               
               if obj.compact, obj.procedure = []; end

           elseif cvfolds > 0 && cvfolds < 1 % percentage
  
               if obj.verbose, fprintf('validating using %.0f%% of the data for testing\n',100*(1-cvfolds)); end                       
                 
               if iscell(design)
                   nclasses = max(design{1});
               else
                   nclasses = max(design);                   
               end

               if iscell(data)

                   traindata = cell(1,length(data));
                   testdata = cell(1,length(data));
                   
                   if iscell(design)
                       traindesign = cell(1,length(data));
                       obj.design = cell(1,length(data));
                   end
                   
                   for c=1:length(data)
                       
                       % make sure classes are evenly represented
                       testidx = [];
                       for k=1:nclasses
                           if iscell(design)
                               tmpidx = find(design{c}(:,1) == k);
                           else
                               tmpidx = find(design(:,1) == k);
                           end
                           testidx = [testidx; tmpidx(1:floor((1-cvfolds)*length(tmpidx)))];
                       end
                       trainidx = ones(1,size(data{c},1)); trainidx(testidx) = 0; trainidx = find(trainidx);
                       
                       % shuffle examples
                       trainidx = trainidx(randperm(length(trainidx)));
                       testidx = testidx(randperm(length(testidx)));

                       traindata{c} = data{c}(trainidx,:);
                       testdata{c} = data{c}(testidx,:);
                       
                       if iscell(design)

                           traindesign{c} = design{c}(trainidx,:);
                           obj.design{c} = design{c}(testidx,:);                           
                       else
             
                           traindesign = design(trainidx,:);
                           obj.design = design(testidx,:);
                       end
                          
                   end

               
               else

                   % make sure classes are evenly represented
                   testidx = [];
                   for k=1:nclasses
                       tmpidx = find(design(:,1) == k);
                       testidx = [testidx; tmpidx(1:floor((1-cvfolds)*length(tmpidx)))];
                   end
                   trainidx = ones(1,size(data,1)); trainidx(testidx) = 0; trainidx = find(trainidx);
                 
                   % shuffle examples
                   trainidx = trainidx(randperm(length(trainidx)));
                   testidx = testidx(randperm(length(testidx)));

                   traindata = data(trainidx,:);
                   testdata = data(testidx,:);

                   traindesign = design(trainidx,:);
                   obj.design = design(testidx,:);

               end

               if (islogical(obj.balanced) && obj.balanced) || strcmp(obj.balanced,'train')
                   if obj.verbose, fprintf('balancing training data\n'); end
                   [testdata,obj.design] = obj.balance(testdata,obj.design);
               end
               
               if (islogical(obj.balanced) && obj.balanced) || strcmp(obj.balanced,'test')
                   if obj.verbose, fprintf('balancing test data\n'); end
                   [traindata,traindesign] = obj.balance(traindata,traindesign);
               end
                  
               assert(~isempty(traindata)  & ~isempty(testdata));
                  
               obj.procedure = obj.procedure.train(traindata,traindesign);
               obj.post = obj.procedure.test(testdata);
               
               if obj.compact, obj.procedure = []; end

           elseif iscell(cvfolds) || isinf(cvfolds) || (cvfolds >= 0 && rem(cvfolds,1) == 0) % n-fold crossvalidation
               
               if iscell(design)
                   nclasses = max(design{1});
               else
                   nclasses = max(design);                   
               end

	       if isinf(cvfolds) % leave-one-out

	        % explicitly specify test indices

	        if iscell(data)
                       nfolds = size(data{1},1);
                else
                       nfolds = size(data,1);
                end
	        cvfolds = num2cell(1:nfolds);

               end
               
               if iscell(cvfolds) % prespecified
                   
                   nfolds = length(cvfolds);
                   
               elseif cvfolds == Inf % leave-one-out crossvalidation 
                   
                   if iscell(data)
                       nfolds = size(data{1},1);
                   else
                       nfolds = size(data,1);
                   end

               else % n-fold
                   
                   nfolds = cvfolds;
               end
 
               % clone the clfproc such that each fold has its own
               if ~iscell(obj.procedure)
                   proc = cell(1,nfolds);
                   
                   for c=1:nfolds
                        proc{c} = obj.procedure;
                   end
                   
                   obj.procedure = proc;
               end

               obj.post = cell(1,nfolds);
               obj.design = cell(1,nfolds);

               for f=1:nfolds

                   if obj.verbose, fprintf('validating fold %d of %d\n',f,nfolds); end                       
                   
                   if iscell(data)

                       traindata = cell(1,length(data));
                       testdata = cell(1,length(data));
                       
                       if iscell(design)
                           traindesign = cell(1,length(data));
                           ftestdesign = cell(1,length(data));
                       end
                       
                       for c=1:length(data)

                           if iscell(cvfolds) % testdata is prespecified
                               testidx = cvfolds{f}{c};
                           else
                               
                               % make sure classes are evenly represented
                               testidx = [];
                               for k=1:nclasses
                                   if iscell(design)
                                       tmpidx = find(design{c}(:,1) == k);
                                   else
                                        tmpidx = find(design(:,1) == k);                                      
                                   end
                                   testidx = [testidx; tmpidx((floor((f-1)*(length(tmpidx)/nfolds))+1):floor(f*(length(tmpidx)/nfolds)))];
                               end
                               
                           end
                           trainidx = ones(1,size(data{c},1)); trainidx(testidx) = 0; trainidx = find(trainidx);
                           trainidx = trainidx(randperm(length(trainidx)));
                           testidx = testidx(randperm(length(testidx)));
                        
                           traindata{c} = data{c}(trainidx,:);
                           testdata{c} = data{c}(testidx,:);

                           if iscell(design)
                               traindesign{c} = design{c}(trainidx,:);
                               ftestdesign{c} = design{c}(testidx,:);
                           else
                               traindesign = design(trainidx,:);
                               ftestdesign = design(testidx,:);
                           end
                       end

                       obj.design{f} = ftestdesign;

                   else
                       
                       if iscell(cvfolds) % testdata is prespecified
                           testidx = cvfolds{f};
                       else
                           % make sure classes are evenly represented
                           testidx = [];
                           for k=1:nclasses
                               tmpidx = find(design(:,1) == k);
                               testidx = [testidx; tmpidx((floor((f-1)*(length(tmpidx)/nfolds))+1):floor(f*(length(tmpidx)/nfolds)))];
                           end
                           %testidx = (floor((f-1)*(size(data,1)/nfolds))+1):floor(f*(size(data,1)/nfolds));
                       end
                       trainidx = ones(1,size(data,1)); trainidx(testidx) = 0; trainidx = find(trainidx);
                       trainidx = trainidx(randperm(length(trainidx)));
                       testidx = testidx(randperm(length(testidx)));

                       traindata = data(trainidx,:);
                       testdata = data(testidx,:);

                       traindesign = design(trainidx,:);
                       
                       obj.design{f} = design(testidx,:);

                   end
                   
                   if (islogical(obj.balanced) && obj.balanced) || strcmp(obj.balanced,'train')
                       if obj.verbose, fprintf('balancing training data\n'); end
                       [traindata,traindesign] = obj.balance(traindata,traindesign);
                   end

                   if (islogical(obj.balanced) && obj.balanced) || strcmp(obj.balanced,'test')
                       if obj.verbose, fprintf('balancing test data\n'); end
                       [testdata,obj.design{f}] = obj.balance(testdata,obj.design{f});
                   end
                  
                   % this might happen if folds end up empty
                   assert(~isempty(traindata)  & ~isempty(testdata));
                  
                   if iscell(obj.procedure)
                       
                       % use a separate procedure per fold
                       % in order to save all produced results
                       obj.procedure{f} = obj.procedure{f}.train(traindata,traindesign);
                       obj.post{f} = obj.procedure{f}.test(testdata);
                       
                       if obj.compact, obj.procedure{f} = []; end;
                       
                   else
                       obj.procedure = obj.procedure.train(traindata,traindesign);
                       obj.post{f} = obj.procedure.test(testdata);
                   end
               end
               
               if ~iscell(obj.procedure) && obj.compact, obj.procedure = []; end
               
           else
               error('unrecognized settting for cvfolds');
           end

       end

   end
end
