classdef validator
%VALIDATOR abstract validator class
%
%   a validator takes a classification procedure
%
%   data is not reshuffled by default by a validator since it may have an inherent
%   temporal ordering (i.e, for dynamic models)
%   
%   Options:
%   'randomize' : reshuffle if true;
%   'procedure' : the classification procedure
%   'verbose'   : output comment if true
%   'balanced'  : balance training data before training a classifier (true,
%                   false,'train', 'test')
%   'compact'   : only retain posteriors and design (in case of memory issues)
%
%   SEE ALSO:
%   crossvalidator.m
%   loocrossvalidator.m
%
%   Copyright (c) 2008, Marcel van Gerven
%
%   $Log: validator.m,v $
%

    properties
        
        post;   % posteriors
        design  % associated class labels
        
        verbose = false;
        randomize = false;
        balanced = false;
        compact = false;
        
        procedure;
    end

    methods
        function obj = validator(varargin)                  
            % a procedure is a clfproc or a cell array of clfprocs
            % the latter is useful for retaining all results in a
            % crossvalidation (although it takes up more space)
                        
            % parse options
            for i=1:2:length(varargin)
                obj.(varargin{i}) = varargin{i+1};
            end

            if isempty(obj.procedure)
                error('classification procedure must be specified!');
            end
            
            if iscell(obj.procedure)
                % convert to clf procedure if input is a cell array
                if obj.verbose
                   fprintf('converting cell array to clfproc\n'); 
                end
                obj.procedure = clfproc(obj.procedure);
            end
            
        end        
        
        function p = significance(obj)
            % Compute significance level that our result is different from
            % another classifier. 
            %
            % Comparison is based on a one-sided binomial test (McNemar)
            % without Bonferroni correction
            %
           
            if iscell(obj.post)
                
                % compute class with highest prior probability
                nclasses = size(obj.post{1},2);
                priors = zeros(1,nclasses);
                for c=1:length(obj.post)
                    for k=1:nclasses
                        priors(k) = priors(k) + sum(obj.design{c}==k);
                    end
                end
                [mxx,class] = max(priors);

                rndpost = cell(1,length(obj.post));
                for c=1:length(obj.post)
                    rndpost{c} = zeros(size(obj.post{c}));
                    rndpost{c}(:,class) = 1;
                end
            else
                
                % compute class with highest prior probability
                nclasses = size(obj.post,2);
                priors = zeros(1,nclasses);
                for k=1:nclasses
                    priors(k) = sum(obj.design==k);
                end
                [mxx,class] = max(priors);

                rndpost = zeros(size(obj.post));
                rndpost(:,class) = 1;
            end
            
            if obj.verbose
               fprintf('performing one-sided biniomial test (p=0.05)\n');
            end
            [r,p,level] = significance(obj.post,obj.design,rndpost,obj.design,'twosided',false);

            if obj.verbose
                
                if r
                    fprintf('null hypothesis rejected (%g<%g);\nsignificant difference from ',p,level);
                else
                    fprintf('null hypothesis not rejected (%g>%g);\nno significant difference from ',p,level);
                end

                if (nargin == 2 && random)
                    fprintf('random classification.\n');
                else
                    fprintf('majority classification (class %d with prior of %g)\n',class,mxx/sum(priors));
                end
            end
        end
        
        function res = getdata(obj,s)
           % returns the results for one dataset in the transfer learning setting 
           % i.e. for the cell array at the lowest level, we retrieve only
           % the results for dataset s

           % TO DO                         
           
        end

        function [result,all] = evaluate(obj,varargin)
           % indirect call to evaluate to simplify the interface 
           
            [result all] = evaluate(obj.post,obj.design,varargin{:});                        
        end
    end    
    
    methods(Access = protected)
        
        function [newdata,newdesign] = balance(obj,data,design)
            % balance data
            
            nclasses = max(design(:,1));
            maxsmp = 0;
            for j=1:nclasses
                summed = sum(design(:,1) == j);
                if ~summed
                   fprintf('not all classes represented in data; refusing to balance\n');
                   newdata = data;
                   newdesign = design;
                   return;
                end
                maxsmp = max(maxsmp,summed);
            end
            newdata = zeros(nclasses*maxsmp,size(data,2));
            for j=1:nclasses
                cdata = data(design(:,1) == j,:);
                if  size(cdata,1) ~= maxsmp
                    
                    % sample with replacement
                    newdata(((j-1)*maxsmp+1):(j*maxsmp),:) = cdata(ceil(size(cdata,1)*rand(maxsmp,1)),:);
                else
                    newdata(((j-1)*maxsmp+1):(j*maxsmp),:) = cdata;
                end
            end
            newdesign=ones(nclasses*maxsmp,1);
            for j=2:nclasses
                newdesign(((j-1)*maxsmp+1):(j*maxsmp)) = j;
            end
            
            % shuffle data
            [newdata,newdesign] = obj.shuffle(newdata,newdesign);
        end
        
        function data = collapse(obj,data)
            % check dimensions of data
            %
            %   Copyright (c) 2008, Marcel van Gerven
            %
            %   $Log: collapse.m,v $
            %
 
            if obj.verbose, fprintf('collapsing data\n'); end
          
            if iscell(data)
                for c=1:length(data)
                    if ndims(data{c}) > 2

                        sz = size(data{c});
                        data{c} = reshape(data{c},[sz(1) prod(sz(2:end))]);
                    end
                end
            else
                if ndims(data) > 2

                    sz = size(data);
                    data = reshape(data,[sz(1) prod(sz(2:end))]);
                end
            end
        end
        
        function [data,design] = shuffle(obj,data,design)
            % shuffle (randomize) examples
            %
            %
            %   Copyright (c) 2008, Marcel van Gerven
            %
            %   $Log: shuffle.m,v $
            %

            if obj.verbose, fprintf('shuffling data\n'); end
          
            if iscell(data)

                % try to keep the same permutation when possible
                prm = randperm(size(data{1},1))';

                if ~iscell(design), design = design(prm,:); end

                for c=1:length(data)

                    sz = size(data{c});

                    if size(prm,1) ~= sz(1)
                        prm = randperm(sz(1));
                    end

                    % first randomize the ordering of the data
                    data{c} = reshape(data{c}(prm,:),sz);

                    if iscell(design)
                        design{c} = design{c}(prm,:);
                    end
                end
            else
                sz = size(data);

                prm = randperm(sz(1));
                data = reshape(data(prm,:),sz);
                design = design(prm,:);
            end
        end

    end
end
